import pandas as pd
import math


def find_distance(cat, owner):

    df_cat = pd.read_csv(cat, sep=",")
    names = df_cat.columns
    df_cat = pd.DataFrame(df_cat, columns=names)

    df_owner = pd.read_csv(owner, sep=",")
    names = df_owner.columns
    df_owner = pd.DataFrame(df_owner, columns=names)

    ds = []

    for i, j in zip(df_cat['indx'], df_owner['indx']):

        p1 = [df_cat['x'][i], df_cat['y'][i]]
        p2 = [df_owner['x'][j], df_owner['y'][j]]
        distance = math.sqrt(((p1[0] - p2[0])**2) + ((p1[1] - p2[1])**2))
        ds.append(distance)

    ds = pd.DataFrame(ds, columns=['distance'])

    df_cat.rename(columns={'x': 'x_cat', 'y': 'y_cat'}, inplace=True)
    df_owner.rename(columns={'x': 'x_owner', 'y': 'y_owner'}, inplace=True)

    result = pd.concat([
        df_cat[{'indx', 'x_cat', 'y_cat'}],
        df_owner[{'x_owner', 'y_owner', 'time', 'trial'}], ds
    ],
                       axis=1)

    result = result[[
        'indx', 'time', 'trial', 'x_cat', 'y_cat', 'x_owner', 'y_owner',
        'distance'
    ]]

    result.to_csv(cat[:-24] + '_improved.csv',
                  index=False,
                  sep=',',
                  encoding='utf-8')



"""Object-specific python package to run automated tasks in the Chase Lab."""

from .ffsync import ffsync
from .ffconcat import ffconcat
from .lazytrim import lazytrim
from .improve_dlc_output import improve_dlc_output
from .find_distance import find_distance
from .improve_dlc_csv import improve_dlc_csv
from .csv2h5 import csv2h5
from .rois import rois

from .version import __version__, VERSION



import json


def lazytrim(trial_times_json):
    def getList(dict):
        list = []
        for key in dict.keys():
            list.append(key)

        return list

    with open(trial_times_json) as json_file:
        trial_times = json.load(json_file)

    catslist = getList(trial_times)

    trials = ['T1', 'T2', 'T3', 'T4', 'T5', 'T6', 'T7', 'T8', 'T9', 'T10']

    for i in catslist:

        d = '#' * 20
        print('\n\n', d, '#' * len(i), d, '\n', d, i, d, '\n', d, '#' * len(i),
              d, '\n')

        for j in trials:

            start = trial_times[i][j][0]
            total = trial_times[i][j][1] - trial_times[i][j][0]
            input_dir = ''
            output_dir = ''
            print('ffmpeg -ss', start, '-i', input_dir + i + '.mp4', '-t',
                  total, '-y', output_dir + i + '_' + j + '.mp4')
            print('&&')



import pandas as pd
from math import sqrt
from numpy import nan

csv_file = '/Users/Felis.catus/Desktop/vars.csv'


def calculateDistance(x1, y1, x2, y2):
    dist = sqrt((x2 - x1)**2 + (y2 - y1)**2)
    return dist


df_cat = pd.read_csv(csv_file, sep=",")

names = df_cat.columns

df_cat = pd.DataFrame(df_cat, columns=names)

df_cat = df_cat.iloc[:, :4]

cat_dst = []

n = len(df_cat)

i = 0

for i in range(0, n):

    x1 = df_cat.iloc[i]['x']
    x2 = df_cat.iloc[i - 1]['x']
    y1 = df_cat.iloc[i]['y']
    y2 = df_cat.iloc[i - 1]['y']

    cat_dst.append(float(calculateDistance(x1, y1, x2, y2)))

cat_dst[0] = nan

cat_dst = pd.DataFrame(cat_dst, columns=['cat_distance'])


def calculateVelocity(dist, time):
    return dist / time


velocity = []

for j in range(0, n):

    if j == 0:
        velocity.append(nan)
        continue

    d = cat_dst.iloc[j]['cat_distance']

    t = df_cat.iloc[j]['time'] - df_cat.iloc[j - 1]['time']

    velocity.append(float(calculateVelocity(d, 0.033)))

velocity = pd.DataFrame(velocity, columns=['velocity'])

moving = []
notMoving = []

for z in cat_dst['cat_distance']:

    if str(z) == 'nan':
        moving.append(nan)
        notMoving.append(nan)
    elif z >= 0.07:
        moving.append(1)
        notMoving.append(0)
    else:
        moving.append(0)
        notMoving.append(1)

moving = pd.DataFrame(moving, columns=['moving'])
notMoving = pd.DataFrame(notMoving, columns=['not_moving'])


def calculateAcc(vi, vf, ti, tf):
    return (vi - vf) / (ti - tf)


acc = []

for q in range(0, n):

    velocity['velocity'].astype('float')

    vi = velocity.iloc[q]['velocity'].astype('float')
    vf = velocity.iloc[q - 1]['velocity'].astype('float')
    ti = df_cat.iloc[q]['time']
    tf = df_cat.iloc[q - 1]['time']

    acc.append(float(calculateAcc(vi, vf, ti, tf)))

acc = pd.DataFrame(acc, columns=['acceleration'])

df = pd.concat([df_cat, cat_dst, velocity, moving, notMoving, acc], axis=1)

print(df)



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['toxopy']

package_data = \
{'': ['*']}

install_requires = \
['pandas', 'dlcu']

setup(name='toxopy',
      version='0.1.5.1',
      description='Object-specific python package to run automated tasks in the Chase Lab.',
      author='fcatus',
      author_email='f.catus@pm.me',
      url='https://github.com/bchaselab/toxopy',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      python_requires='>=3.6',
     )



import os
from subprocess import run
import pandas as pd
import shlex


def ffsync(sync_csv, videos_dir):

    df = pd.read_csv(sync_csv, sep=",")

    names = df.columns
    df = pd.DataFrame(df, columns=names)

    camera_1 = df.iloc[0][0]
    camera_2 = df.iloc[1][0]

    offset_1 = df.iloc[0][1]
    offset_2 = df.iloc[1][1]

    offset = []

    if offset_1 != 0:
        offset.append(camera_1)
        offset.append(offset_1)
        offset.append(camera_2)
    elif offset_2 != 0 and offset_2 > 0:
        offset.append(camera_2)
        offset.append(offset_2)
        offset.append(camera_1)
    elif offset_2 != 0 and offset_2 < 0:
        offset.append(camera_1)
        offset.append(offset_2 * -1)
        offset.append(camera_2)

    ffprobe_command = 'ffprobe -v error -show_entries format=duration -of default=noprint_wrappers=1:nokey=1 ' + videos_dir + '/'

    ffprobe_1 = ffprobe_command + offset[0]
    ffprobe_2 = ffprobe_command + offset[2]

    duration_values = []

    for i in [ffprobe_1, ffprobe_2]:
        ffprobe_run = os.popen(i)
        video_duration = ffprobe_run.read()
        video_duration = video_duration.strip('\n')
        duration_values.append(video_duration)

    end_time = min(duration_values)

    ffmpeg_run = shlex.split('ffmpeg -ss ' + str(offset[1]) + ' -i ' +
                             videos_dir + '/' + str(offset[0]) + ' -t ' +
                             end_time + ' -y ' + videos_dir + '/synced_' +
                             str(offset[0]))

    run(ffmpeg_run)



import warnings
warnings.filterwarnings("ignore")

import pandas as pd
from pathlib import Path
import numpy as np
import os
import json
import matplotlib.pyplot as plt
from dlcu import PlottingResults, time_in_each_roi
from collections import namedtuple
import matplotlib.patches as patches



def rois(cat, improved_file):
# Load dataset

    Dataframe = pd.read_csv(improved_file)
    df = Dataframe.dropna()

    # Create variable names

    time = df['time']
    x_cat = df['x_cat_loess']
    y_cat = df['y_cat_loess']
    velocity = df['velocity_loess']

    # Plot velocity

    plt.plot(time, velocity)
    plt.xlabel('Time in seconds')
    plt.ylabel('Speed in pixels per second')
    plt.savefig(cat + '_velocity.png')
    plt.close()

    # Calculate and plot rois


    def rois_gen(i):
        def getList(dict):
            list = []
            for key in dict.keys():
                list.append(key)

            return list

        with open('room_layout.json') as json_file:
            p = json.load(json_file)

        catslist = getList(p)

        positions = ['middle', 'rightside', 'leftside', 'topside', 'bottomside']

        colors = ['orange', 'purple', 'red', 'blue', 'green']

        position = namedtuple('position', ['topleft', 'bottomright'])

        #two points defining each roi: topleft(X,Y) and bottomright(X,Y).

        return {
            'middle':
            position((p[i]['middle_tl'][0], p[i]['middle_tl'][1]),
                     (p[i]['middle_br'][0], p[i]['middle_br'][1])),
            'rightside':
            position((p[i]['right_tl'][0], p[i]['right_tl'][1]),
                     (p[i]['right_br'][0], p[i]['right_br'][1])),
            'leftside':
            position((p[i]['left_tl'][0], p[i]['left_tl'][1]),
                     (p[i]['left_br'][0], p[i]['left_br'][1])),
            'topside':
            position((p[i]['top_tl'][0], p[i]['top_tl'][1]),
                     (p[i]['top_br'][0], p[i]['top_br'][1])),
            'bottomside':
            position((p[i]['bottom_tl'][0], p[i]['bottom_tl'][1]),
                     (p[i]['bottom_br'][0], p[i]['bottom_br'][1]))
        }


    bp_tracking = np.array((x_cat, y_cat, velocity))

    rois = rois_gen(cat)

    fig, ax = plt.subplots(1)

    plt.plot(x_cat, y_cat, '.-')

    positions = ['middle', 'rightside', 'leftside', 'topside', 'bottomside']
    colors = ['orange', 'purple', 'red', 'blue', 'green']

    for position, color in zip(positions, colors):

        rect = patches.Rectangle(
            rois[position].topleft,
            rois[position].bottomright[0] - rois[position].topleft[0],
            rois[position].bottomright[1] - rois[position].topleft[1],
            linewidth=1,
            edgecolor=color,
            facecolor='none')
        ax.add_patch(rect)

    plt.ylim(-100, 600)
    plt.savefig(cat + '_rois.png')
    plt.close()

    res = time_in_each_roi.get_timeinrois_stats(bp_tracking.T,
                                                rois,
                                                fps=30,
                                                returndf=True,
                                                check_inroi=True)

    res.to_csv(improved_file[:-4] + '_rois.csv',
              index=False,
              sep=',',
              encoding='utf-8')
